@@ -1,7 +1,8 @@
+from django.contrib.postgres.fields import JSONField
 from django.db.models.aggregates import Aggregate
 
 __all__ = [
-    'ArrayAgg', 'BitAnd', 'BitOr', 'BoolAnd', 'BoolOr', 'StringAgg',
+    'ArrayAgg', 'BitAnd', 'BitOr', 'BoolAnd', 'BoolOr', 'JSONBAgg', 'StringAgg',
 ]
 
 
@@ -30,12 +31,23 @@ class BoolOr(Aggregate):
     function = 'BOOL_OR'
 
 
+class JSONBAgg(Aggregate):
+    function = 'JSONB_AGG'
+    _output_field = JSONField()
+
+    def convert_value(self, value, expression, connection, context):
+        if not value:
+            return []
+        return value
+
+
 class StringAgg(Aggregate):
     function = 'STRING_AGG'
-    template = "%(function)s(%(expressions)s, '%(delimiter)s')"
+    template = "%(function)s(%(distinct)s%(expressions)s, '%(delimiter)s')"
 
-    def __init__(self, expression, delimiter, **extra):
-        super(StringAgg, self).__init__(expression, delimiter=delimiter, **extra)
+    def __init__(self, expression, delimiter, distinct=False, **extra):
+        distinct = 'DISTINCT ' if distinct else ''
+        super(StringAgg, self).__init__(expression, delimiter=delimiter, distinct=distinct, **extra)
 
     def convert_value(self, value, expression, connection, context):
         if not value:
