@@ -15,7 +15,9 @@ class Migration(migrations.Migration):
             name='Site',
             fields=[
                 ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
-                ('domain', models.CharField(max_length=100, verbose_name='domain name', validators=[_simple_domain_name_validator])),
+                ('domain', models.CharField(
+                    max_length=100, verbose_name='domain name', validators=[_simple_domain_name_validator]
+                )),
                 ('name', models.CharField(max_length=50, verbose_name='display name')),
             ],
             options={
